@@ -32,7 +32,7 @@ def scd(
     :param source: The source data that will be used to merge with `delta_table`
     :param keys: Column name(s) that identify unique rows. Can be a single column name as a string, or a list of strings, where order of the list does not matter.
     :param effective_ts: The name of the existing column that will be used as the “start” or “effective” indicator for a given entity. The column be of any orderable type, including timestamp, date, string, and numeric types.
-    :param end_ts: The name of the non-existing column that will be used as the “end” indicator for a given entity. Its type will match the type of effective_ts.
+    :param end_ts: Only required for type 2. The name of the non-existing column that will be used as the “end” indicator for a given entity. Its type will match the type of effective_ts.
     :param scd_type: The type of SCD that is to be performed
     :return: The same `delta_table`
 
@@ -202,11 +202,11 @@ def bootstrap_scd2(
 
         import hydro.delta as hd
         delta_table = hd.bootstrap_scd2(
-        df,
-        keys=["id"],
-        effective_ts="date",
-        end_ts="end_date",
-        path="/path/to/delta/table",
+            df,
+            keys=["id"],
+            effective_ts="date",
+            end_ts="end_date",
+            path="/path/to/delta/table",
         )
 
     Results in:
@@ -233,12 +233,15 @@ def bootstrap_scd2(
         raise ValueError(
             'Need to specify one (or both) of `path` and `table_identifier`',
         )
+
+    # scd2-ify the existing data
     window = Window.partitionBy(keys).orderBy(effective_ts)
     final_payload = source_df.withColumn(
         end_ts,
         F.lead(effective_ts).over(window),
     )
 
+    # build the DeltaTable object
     builder = DeltaTable.createOrReplace(
         source_df.sparkSession,
     )  # TODO change to createIfNotExists?
@@ -253,7 +256,9 @@ def bootstrap_scd2(
         builder = builder.location(path)
     if table_identifier:
         builder = builder.tableName(table_identifier)
-    builder.execute()
+    builder.execute()  # save DeltaTable to disk
+
+    # write data to disk
     if table_identifier:
         final_payload.write.format('delta').option('mergeSchema', 'true').mode(
             'append',
@@ -285,25 +290,31 @@ def deduplicate(
     """
     if isinstance(keys, str):  # pragma: no cover
         keys = [keys]
-    detail_object = detail(delta_table)
-    target_location = detail_object['location']
-    table_version = detail_object['version']
+    detail_object = _DetailOutput(delta_table)
+    target_location = detail_object.location
+    table_version = delta_table.history().select('version').limit(1).collect()[0].asDict()['version']
     print(
         f'IF THIS OPERATION FAILS, RUN RESTORE TO {table_version} WITH deltaTable.restoreToVersion({table_version})',
     )
     df = delta_table.toDF()
     spark = df.sparkSession
 
+    # Deduplicate the dataframe using `hydro.spark.deduplicate_dataframe`
     deduped = hs.deduplicate_dataframe(df, keys=keys, tiebreaking_columns=tiebreaking_columns)
     deduped.write.format('delta').save(backup_path)
 
+    # To delete matching rows, we use MERGE. MERGE requires a condition that matches rows between source and target
     merge_key_condition = ' AND '.join(
         [f'source.{key} = target.{key}' for key in keys],
     )
+
+    # Perform deletion of existing duplicates using MERGE
     delta_table.alias('target').merge(
         deduped.select(keys).distinct().alias('source'),  # do we need distinct here?
         merge_key_condition,
     ).whenMatchedDelete().execute()
+
+    # Insert the de-duplicated data into the target Delta table
     spark.read.format('delta').load(backup_path).write.format('delta').mode(
         'append',
     ).save(target_location)
@@ -320,10 +331,10 @@ def partial_update_set(
 
     Partial updates in this case are when some columns in the data are NULL, but are meant to be non-destructive, or is there no semantic meaning to the NULLs.
 
-    :param delta_frame: A Delta Lake table or DataFrame that describes a source dataframe
+    :param delta_frame: A DeltaTable or DataFrame that describes a source MERGE dataframe
     :param source_alias: A temporary name given to the source data of the MERGE
     :param target_alias: A temporary name given to the target Delta Table of the MERGE
-    :return: A dictionary that describes non-destructive updates for all fields in `delta_frame`
+    :return: A dictionary that describes non-destructive updates for all fields in `delta_frame` in {key: coalesce(source.key, target.key)} form
 
     Example
     -----
@@ -473,6 +484,7 @@ def detail(delta_table: DeltaTable) -> dict[str, Any]:
     allfiles = _snapshot_allfiles(delta_table)
     partition_columns = [f'partitionValues.{col}' for col in details['partition_columns']]
 
+    # compute the number of records in the current snapshot by scanning the Delta log
     num_records = (
         allfiles.select(
             F.get_json_object('stats', '$.numRecords').alias('num_records'),
@@ -482,6 +494,7 @@ def detail(delta_table: DeltaTable) -> dict[str, Any]:
     )
     details['numRecords'] = _humanize_number(num_records)
 
+    # compute the percentage of files that have statistics in the current snapshot by scanning the Delta log
     stats_percentage = (
         allfiles.agg(
             F.avg(
@@ -491,19 +504,21 @@ def detail(delta_table: DeltaTable) -> dict[str, Any]:
     ).collect()[0]['stats_percentage']
     details['stats_percentage'] = stats_percentage * 100
 
+    # compute the number of partitions in the current snapshot by scanning the Delta log
     partition_count = allfiles.select(*partition_columns).distinct().count()
     details['partition_count'] = _humanize_number(partition_count)
 
+    # compute the version of the current snapshot by looking at the table's history, which scans the Delta log
     version = delta_table.history().select('version').limit(1).collect()[0].asDict()['version']
     details['version'] = _humanize_number(version)
     return details
 
 
 def _snapshot_allfiles(delta_table: DeltaTable) -> DataFrame:
-    # this is kinda hacky but oh well
     spark = delta_table.toDF().sparkSession
     location = delta_table.detail().collect()[0]['location']
 
+    # py4j nonsense to call the DeltaLog API
     delta_log = spark._jvm.org.apache.spark.sql.delta.DeltaLog.forTable(
         spark._jsparkSession,
         location,
